# Demonstrate how to triage, debug and fix INT8 accuracy issue


This sample is modified based on [sampleMNIST](https://github.com/NVIDIA/TensorRT/tree/release/5.1/samples/opensource/sampleMNIST). 


## Why We Need It?

Refer to [INT8 Accuracy FAQ](https://elinux.org/TensorRT) to understand the background and procedure.


## How To Build It?

Adjust the CUDA and TensorRT path of Makefile before compiling it.

```
CUDA_INSTALL_PATH ?= /usr/local/cuda-10.1
TRT_INSTALL_PATH ?= /home/nfeng/tensorrt/v5.1_ga/TensorRT-5.1.5.0
```
NOTE: If TensorRT is installed into system directory, there is no need to adjust its path.


## How To Run It?

Run FP32 inference,
```
./sample_mnist
```
It dumps the network output (prob layer for mnist) into _./result/fp32-prob.txt_ folder.

Run INT8 inference,
```
./sample_mnist --int8 --scopedLayerIndex 5
```
It sets layer [0~5] running INT8 mode and layer (5, 11] running FP32 mode, and then dump the network output. 
```
./sample_mnist --int8 --layerStickToFloat 5
```
It sets only layer 5 running FP32 mode and the other layers are still in INT8 mode if INT8 implementation for them exit, and then dump the network output. 


## How To Use layer_analyzer.py?

```
python layer_analyzer.py -a ./sample_mnist -n 9 -o prob -d ./results/ -m 1 -t 0.98
```
NOTE: 
* -a denotes where the inference application locates.
* -n denotes how many layers we are going to scope.
* -o denotes the output name of your network.
* -d denotes the folder used to store the dumpped files.
* -m denotes the comparison metrics of two output tensors.
* -t denotes the threshold used to determine whether current scoped layer is loss layer.
